*
* Filename SCTB.INC
*
* Changes after Version 1.30 release:
*
* 25.06.90 rr  SC_CHECK braucht keinen Supervisor-Mode mehr
* 26.06.90 rr  SC_CHECK liefert in A0 die Toolbox-Trap4-Adresse
*
*

******************************************************************
*                                                                *
*         Dieser Source-Code darf nicht modifiziert, erweitert   *
*         oder gekrzt werden. Das Benutzerprogramm soll diese   *
*         Datei durch eine INCLUDE-Anweisung einlesen.           *
*         Nur so kann gewhrleistet werden, da Ihre Programme   *
*         auch mit zuknftigen Versionen der                     *
*                                                                *
*                            SuperCharger Toolbox                *
*                                                                *
*         kompatibel sind. Falls diese Vorgehensweise fr Ihre   *
*         Applikation nicht mglich ist, wenden Sie sich bitte   *
*         schriftlich an                                         *
*                                                                *
*              Beta Systems AG                                   *
*              Toolbox Support                                   *
*              Stauffenstrae 42                                 *
*              6000 Frankfurt                                    *
*                                                                *
******************************************************************

*                        FUNCTION CALLS
*
* Aufruf duch TRAP 4 mit dem Funktionscode in Register D0. Die anderen
* Register mssen wie unten angegeben geladen sein. Fr genauere
* Informationen siehe Dokumentation der SuperCharger Toolbox.
*

SC_STAT   equ  $0002 * GET STATUS       D1=DEVICE
SC_BOOT   equ  $0003 * BOOT TOOLBOX     D1=DEVICE
SC_SD     equ  $0004 * SEND DATA        D1=DEVICE,D2=SIZE,           A0=ADDR
SC_GD     equ  $0005 * GET DATA         D1=DEVICE,                   A0=ADDR
SC_SM     equ  $0006 * SEND MEMORY      D1=DEVICE,D2=SIZE,D3=SC_ADDR,A0=ADDR
SC_GM     equ  $0007 * GET MEMORY       D1=DEVICE,D2=SIZE,D3=SC_ADDR,A0=ADDR
SC_EXEC   equ  $0008 * EXECUTE PRG.     D1=DEVICE,        D3=SC_ADDR
SC_SB     equ  $0009 * SEND 1 BYTE      D1=DEVICE,D2=DATA
SC_GB     equ  $000A * GET 1 BYTE       D1=DEVICE (returns Byte in D2)
SC_LLCAD  equ  $000B * LLC ADDRESS DMA  D1=DEVICE
SC_LLCSB  equ  $000C * LLC Send Byte    D1=DEVICE,D2=DATA
SC_LLCGB  equ  $000D * LLC Get Byte     D1=DEVICE (returns Byte in D2)
SC_LLCTO  equ  $000E * LLC Tooltransout D1=DEVICE,D2=SIZE,           A0=ADDR
SC_LLCTI  equ  $000F * LLC Tooltransin  D1=DEVICE,D2=SIZE,           A0=ADDR



* --------------------------------------------------------------------
*
*                             SC_CHECK  Check Toolbox
*
* FUNKTION: Prft, ob der residente Teil der Toolbox geladen ist.
*
* INPUT   : -none-
*
* OUTPUT  : D0 = Result Code
*           D1 = Versionsnummer der Toolbox als Ascii-Langwort
*           A0 = Startadresse des Toolbox-Trap #4
*
* HINWEIS : Diese Funktion wird im Source-Code angegeben, da sie das
*           Vorhandensein der Toolbox berprft und somit nicht
*           Bestandteil der Toolbox sein darf.
*

trap4vec  equ  $90                      * Trap 4 Vektor
xbraid    equ  $58425241                * XBRA Identification
sctbid    equ  $53435442                * SCTB Identification

          BRA       SC_AROUND

SC_CHECK  move.l    #-1,-(sp)
          move.w    #36,-(sp)           * Get Trap 4 Vektor
          move.w    #5,-(sp)            * setexec
          trap      #13
          addq.l    #8,sp
          move.l    d0,a0               * trap 4 vector in d0 and a0
          cmp.l     #xbraid,-12(a0)     * Check XBRA ID
          bne       sc_chk01
          cmp.l     #sctbid,-8(a0)      * Check SCTB ID
          bne       sc_chk01

          moveq     #0,d0               * Result Code
          move.l    -16(a0),d1          * Version Number
          rts

sc_chk01  moveq     #0,d1               * Version Number n/a
          moveq     #-1,d0              * Result Code -1
          rts

SC_AROUND nop

* --------------------------------------------------------------------
*
*                   RESULT CODES
*                   ============
*
*    0  OK, kein Fehler aufgetreten
*   -1  Residente Toolbox nicht geladen
*   -2  Reserved
*   -3  SuperCharger nicht angeschlossen
*   -4  Timeout (SuperCharger-Programm antwortet nicht)
*   -5  Protokollfehler (z.B. Daten erwartet, Speicherblock empfangen)
*   -6  ABIO-Hotkey erkannt (siehe SC_CHECK)
*   -7  Reserved
*   -8  Illegal function call
*
* --------------------------------------------------------------------


sce_okay  equ  0    * okeeh
sce_na    equ  -1   * Toolbox not available (SCTB.PRG not resident)
sce_nc    equ  -3   * SuperCharger not connected
sce_to    equ  -4   * Timeout (no response from SuperCharger program)
sce_pf    equ  -5   * Protocol failure
sce_abio  equ  -6   * ABIO hotkey detected
sce_if    equ  -8   * Illegal function call

