*
* File SC_RAM.S Version 1.30
*
* Macht den Speicher des SuperChargers unter TOS als Ramdisk zugnglich.
* Programmiert unter Verwendung der SuperCharger ToolBox.
*
* SC_RAM.TTP 3G
*            !!
*            !+---- Bevorzugte Laufwerkskennung. Falls schon belegt, wird
*            !      die erste freie Laufwerkskennung verwendet. Wenn nichts
*            !      angegeben wird, ist I: die bevorzugte Laufwerkskennung.
*            !
*            +----- DMA Gerteadresse. Default ist "3"
*
*
* Wird SC_RAM.TTP nach SC_RAM.PRG umbenannt, so erfolgt die Initialisierung
* mit Defaultwerten, also Gerteadresse 3 und bevorzugte Laufwerkskennung
* "I:".
*

* Systemvariablen

hdv_bpb   equ  $472                * Bios Parameter Block Routine
hdv_rw    equ  $476                * Read/Write Routine
hdv_med   equ  $47e                * Mediachange Routine
drvbits   equ  $4c2                * Bitvektor aktive Laufwerke
hz200     equ  $4ba                * 200 Hz Timer

* Eigene Konstanten

err_drv   equ  $01                 * "more than 32 drives"
err_aktiv equ  $02                 * "ABIO Aktiv" Meldung
err_retry equ  $03                 * "Retry, Abort?" Meldung
signup    equ  $04                 * Signup Message
warmstart equ  $05                 * Warmstart Message

* Befehlscodes zur Kommunikation mit dem V30-Programmteil

cmd_reset equ  $0000               * Reset Ramdisk, lsche Speicher
cmd_size  equ  $0001               * Get Ramdisk Size
cmd_read  equ  $0002               * Read sector(s)
cmd_write equ  $0003               * Write sector(s)


start     jmp       Install        * Ramdisk resident installieren

          INCLUDE   SCTB_INC.S     * Toolbox Softwareschnittstelle


*---------------------------------------------------------------------
*                   RAMDISK KERNAL: GET BIOS PARAMETER BLOCK
*---------------------------------------------------------------------

* Liefert den Bios Parameter Block zu einem angegebenen Laufwerk. Es
* wird geprft, ob die Ramdisk angesprochen wurde. Wenn nicht, wird
* zum bisherigen BPB-Vektor verzweigt. Sonst wird der Ramdisk-BPB
* geliefert.


          dc.b      'XBRA'         * XBRA-Kennung
          dc.b      'SCRD'         * Programm-Kennung "SCRD"
old_bpb   ds.l      1              * Alter BPB-Vektor
my_bpb    move.w    drive_id,d0    * Ramdisk Laufwerkskennung
          cmp.w     4(sp),d0       * Ramdisk angesprochen?
          beq       our_bpb
          move.l    old_bpb,a0     * Alten Vektor anspringen
          jmp       (a0)

our_bpb   move.l    #bpb,d0        * Liefert den Ramdisk-BPB zurck
          rts                      * das war's schon!


*---------------------------------------------------------------------
*                   RAMDISK KERNAL: MEDIA CHANGE
*---------------------------------------------------------------------

* Prft, ob das Medium gewechselt wurde. Da beim SuperCharger ein
* Austausch whrend des Betriebs nicht vorgesehen ist, wird immer der
* Wert "Kein Wechsel" geliefert.

          dc.b      'XBRA'         * XBRA-Kennung
          dc.b      'SCRD'         * SuperCharger Ramdisk
old_med   ds.l      1
my_med    move.w    drive_id,d0    * Ramdisk Laufwerkskennung
          cmp.w     4(sp),d0       * Ramdisk angesprochen?
          beq       our_med
          move.l    old_med,a0     * Alten Vektor anspringen
          jmp       (a0)

our_med   moveq.l   #0,d0          * Kein Media Change
          rts


*---------------------------------------------------------------------
*                   RAMDISK KERNAL: READ/WRITE
*---------------------------------------------------------------------

* Sektoren lesen und schreiben.


          dc.b      'XBRA'         * XBRA-Kennung
          dc.b      'SCRD'         * SuperCharger Ramdisk
old_rw    ds.l      1
my_rw     move.w    drive_id,d0    * Ramdisk Laufwerkskennung
          cmp.w     14(sp),d0      * Ramdisk angesprochen?
          beq       our_rw
          move.l    old_rw,a0      * Alten Vektor anspringen
          jmp       (a0)

our_rw    move.w    12(sp),sektor  * Startsektor
          move.w    10(sp),anzahl  * Anzahl der Sektoren
          beq       Fertig         * wenn 0 Sektoren, fertig
          move.l    6(sp),startadr * Startadresse
          btst      #0,5(sp)       * Lesen oder schreiben?
          beq       our_read
          bra       our_write


* Sektoren lesen. Jeder Sektor wird einzeln eingelesen.

our_read  moveq.b   #sc_sb,d0      * sende Auftragscode als Byte
          move.w    dev_id,d1
          moveq.b   #cmd_read,d2   * Befehlscode an V30-Programm schicken
          trap      #4
          bmi       retry          * Fehlerbehandlung

          moveq.b   #sc_sb,d0      * Sende Einzelbyte
          move.w    dev_id,d1
          move.b    sektor+1,d2    * Sende Sektor LSB
          trap      #4
          bmi       retry

          moveq.b   #sc_sb,d0      * Sende Einzelbyte
          move.w    dev_id,d1
          move.b    sektor,d2      * Sende Sektor MSB
          trap      #4
          bmi       retry

          moveq.b   #sc_gd,d0      * empfange Daten
          move.w    dev_id,d1
          move.l    #$00000200,d2  * Lnge 512 Bytes
          move.l    startadr,a0    * Datenadresse
          trap      #4             * Toolbox
          bmi       retry          * Fehlerbehandlung

          add.l     #512,startadr  * Startadresse hochzhlen
          addq.w    #1,sektor
          subq.w    #1,anzahl      * Sektorzhler herunterzhlen
          bne       our_read       * weitermachen, bis alle Sektoren fertig.

fertig    moveq     #0,d0          * Kein Fehler
          rts                      * Ende OUR_READ


our_write moveq.b   #sc_sb,d0      * sende Auftragscode als Byte
          move.w    dev_id,d1
          moveq.b   #cmd_write,d2  * Befehlscode an V30-Programm schicken
          trap      #4
          bmi       retry          * Fehlerbehandlung

          moveq.b   #sc_sb,d0      * Sende Einzelbyte
          move.w    dev_id,d1
          move.b    sektor+1,d2    * Sende Sektor LSB
          trap      #4
          bmi       retry

          moveq.b   #sc_sb,d0      * Sende Einzelbyte
          move.w    dev_id,d1
          move.b    sektor,d2      * Sende Sektor MSB
          trap      #4
          bmi       retry

          moveq.b   #sc_sd,d0      * Sende Daten
          move.w    dev_id,d1
          move.l    #$00000200,d2  * Lnge 512 Bytes
          move.l    startadr,a0    * Datenadresse
          trap      #4             * Toolbox
          bmi       retry          * Fehlerbehandlung

          add.l     #512,startadr  * Startadresse hochzhlen
          addq.w    #1,sektor      * Nchster Sektor
          subq.w    #1,anzahl      * Sektorzhler herunterzhlen
          bne       our_write      * weitermachen, bis alle Sektoren fertig.
          bra.s     fertig

* Fehlerbehandlung. Es wird eine Fehlermeldung ausgegeben, danach der Text
* "R=RETRY, A=ABORT -->'. Die entsprechende Aktion wird dann ausgefhrt.


retry     bsr       message        * Fehlermeldung ausgeben
          move.w    #err_retry,d0  * "retry, abort?"
          bsr       message

retry1    bsr       getkey         * auf Tastendruck warten
          cmp.b     #'R',d0        * retry?
          beq       our_rw         * dann eben nochmal!
          cmp.b     #'A',d0        * abort?
          bne       retry1         * wenn nicht, auf neue Taste warten

          moveq.l   #-1,d0         * "allgemeiner Fehler"
          rts



*=======================================
*
* Installiere Ramdisk im Atari
*
*

install:  move.l    4(a7),a5       * Basepage Adresse nach A5
          move.l    a5,basepage    * Basepage Adresse merken

* Eigenen Speicherbedarf ermitteln

          move.l    12(a5),d0      * text
          add.l     20(a5),d0      * data
          add.l     28(a5),d0      * bss
          add.l     #$100,d0       * basepage
          move.l    d0,mysize      * Lnge des reservierten Atari-Speichers

* Stackpointer auf Programmende + $0400 setzen

          add.l     #$400,d0       * Stack whrend der Installation
          move.l    d0,d1
          add.l     a5,d0          * Endadresse in D0
          bclr.l    #0,d0          * Adresse gerade machen
          move.l    d0,sp          * eigenen Stackpointer setzen

* Restlichen Speicher freigeben

          move.l    d1,-(sp)       * Lnge des Programms+Stack+Basepage
          move.l    a5,-(sp)       * Ab Basepage
          clr.w     -(sp)
          move.w    #$4a,-(sp)     * SETBLOCK: Speicherbereich reservieren
          trap      #1
          add.l     #12,sp


* Da im Folgenden auf Systemvariablen zugegriffen wird, mu in den
* Supervisor Mode umgeschaltet werden.

          clr.l     -(sp)          * 
          move.w    #32,-(sp)      * GemDos: SUPERVISOR einschalten
          trap      #1
          addq.l    #6,sp
          move.l    d0,ssave       * User Stackpointer sichern


* Auswertung der Befehlszeile.

          move.l    basepage,a0    * Basepage Adresse nach A0
          move.b    $80(a0),d0     * Commandline length nach D0 holen
          beq       init00         * Defaults verwenden, INIT anspringen

* Erstes Zeichen: DMA-Adresse. Erlaubter Bereich: "0" bis "7". Default: "3"

          moveq     #0,d1
          move.b    $81(a0),d1     * Hole DMA-Devicenummer
          sub.b     #'0',d1
          bcs       init00         * Bereich 0..7 prfen
          cmp.b     #'7'-'0'+1,d1
          bcc       init00
          move.w    d1,dev_id      * Device ID sichern
          cmp.b     #1,d0          * Nur ein Parameter gegeben?
          beq       init00         * Dann sind wir fertig

* 2. Zeichen: Bevorzugte Laufwerkskennung. Erlaubter Bereich: "A" bis "P".

          move.b    $82(a0),d1     * Hole Laufwerkskennung
          and.w     #$5F,D1        * Kleinschrift zu Groschrift
          sub.b     #'A',d1
          bcs       init00         * Bereich A..Z prfen
          cmp.b     #'P'-'A'+1,d1
          bcc       init00
          move.w    d1,drive_id    * Default Laufwerkskennung speichern.


* Prfe, ob der residente Teil der Toolbox geladen wurde. Falls nicht,
* erfolgt eine entsprechende Fehlermeldung.

init00    bsr       sc_check       * Prfe, ob Toolbox vorhanden
          bmi       abort          * wenn nicht, abbrechen.


* Nun wird mittels SC_STAT geprft, ob bereits die Toolbox im SuperCharger
* aktiv ist. Wird ein ABIO.TOS Hotkey erkannt, erfolgt eine Fehlermeldung.
* wird eine Toolbox erkannt, so wird davon ausgegangen, da die Ramdisk
* bereits aktiv ist (z.B. nach einem Atari-Reset). In diesem Falle wird
* der SuperCharger-Speicher nicht gelscht, damit der Inhalt der Ramdisk
* erhalten bleibt.


init01    move.w    #sc_stat,d0    * prfe Status der Toolbox
          move.w    dev_id,d1
          trap      #4
          bne       init01a        * Wenn nicht Toolbox, weiterprfen
          move.w    #warmstart,d0  * Warmstart-Meldung ausgeben
          bsr       message
          bra       init04         * Kein Reset ausfhren

init01a   cmp.w     #sce_abio,d0   * abio hotkey aktiv?
          bne       init03         * jede andere Meldung fhrt zum Kaltstart

* Ein ABIO Hotkey wurde erkannt. Der Benutzer kann nun whlen zwischen
* Abbruch (Ramdisk wird nicht installiert und Hotkey bleibt erhalten) und
* Reset (Ramdisk berschreibt ABIO Hotkey).

          move.w    #err_aktiv,d0  * "Abio hotkey aktiv"
          bsr       message

init02    bsr       getkey
          cmp.b     #'R',D0
          beq       init03         * [R]eset --> Kaltstart
          cmp.b     #'A',D0
          bne       init02         * Wenn nicht [A]bbruch, dann Schleife

          move.w    #sce_abio,d0   * Identifikation fr Abio-Fehler
          bra       abort  


* Der SuperCharger wird zurckgesetzt, das Ramdisk-Programm wird geladen
* und gestartet (Kaltstart mit Lschen des SuperCharger-RAM).

init03    move.w    #sc_boot,d0    * Boote SuperCharger Toolbox
          move.w    dev_id,d1
          trap      #4
          bmi       abort          * Fehlerbehandlung, falls ntig

          move.w    #sc_sm,d0      * send memory (Ramdisk Programm)
          move.w    dev_id,d1      * DMA-Gerteadresse
          move.l    #v30_size,d2   * Gre des V30-Programms
          move.l    #v30_addr,d3   * Zieladresse im Supercharger
          lea       v30_start,a0   * Quelladresse im Atarispeicher
          trap      #4
          bmi       abort          * Fehlerbehandlung

          move.w    #sc_exec,d0    * Starte Ramdiskprogramm im SuperCharger
          move.w    dev_id,d1
          move.l    #v30_addr,d3   * Startadresse des V30 Programms
          trap      #4
          bmi       abort  


* Der nun aktivierten SuperCharger-Ramdisk wird der Befehl RESET gegeben,
* um den Speicher zu lschen.

          moveq.b   #sc_sb,d0      * Send Byte
          move.w    dev_id,d1
          moveq.b   #cmd_reset,d2  * Lsche den Speicher im SuperCharger
          trap      #4
          bmi       abort

* Die Gre des verfgbaren RAM-Speichers im SuperCharger wird erfragt,
* der Wert in den Bios Parameter Block BPB eingetragen.


init04    moveq.b   #sc_sb,d0      * Send Data Byte
          move.w    dev_id,d1
          moveq.b   #cmd_size,d2   * Befehl "Get Size" ans V30-Programm
          trap      #4
          bmi       abort          * Falls Fehler, abbrechen.

* Als Antwort auf den Befehl CMD_SIZE sendet das V30-Programm ein Byte
* zurck. $01 bedeutet 512KB SuperCharger, $00 bedeutet 1 MB SuperCharger.


          moveq.b   #sc_gb,d0      * Get Data Byte
          move.w    dev_id,d1
          trap      #4
          bmi       abort

* Hinweis zur Berechnung der freien Cluster:
*
* Im SuperCharger-Speicher sind $8000 Bytes (32k) reserviert fr Toolbox
* und Programm. Somit ergeben sich folgende Rechnungen:
*
* 1 MB-Gert: $F8000 freier Speicher / $200 Sektorlnge = 1984 Sektoren.
* 512k-Gert: $78000 freier Speicher / $200 Sektorlnge =  960 Sektoren.
*
* Davon werden 18 Sektoren fr 2 FATs und das Directory abgezogen
* (siehe BPB-Definition). Die verbleibende Zahl wird durch 2 geteilt,
* da ein Cluster aus 2 Sektoren besteht. Es ergeben sich 983 freie
* Datencluster fr 1 MB, sowie 471 freie Datencluster fr 512 KB.

          cmp.b     #$01,d2        * Bei Antwort $01 springe nach SC512K
          beq       sc512k
          move.w    #983,bpb_size  * 983 Datencluster bei 1 MB
          bra.s     sc1meg
sc512k    move.w    #471,bpb_size  * 471 Datencluster bei 512k
sc1meg    nop

* Die Initialisierung des SuperChargers ist nun beendet. Es folgt die
* Einbindung der Ramdisk in das Atari-Betriebssystem.

* Suche nach geeigneter Laufwerksbezeichnung.

          move.l    drvbits,d0     * Bitvektor der aktiven Laufwerke
          move.w    drive_id,d4    * Bevorzugtes Laufwerk nach D4 laden
          bset      d4,d0          * Entsprechendes Bit setzen
          beq       init06         * Wenn das Bit vorher 0 war, war's das.

          moveq.l   #0,d4          * Sonst ab A: nach freier Kennung suchen
          moveq.l   #31,d5         * und nach 32 Tests abbrechen

init05    bset      d4,d0          * Bit testen und dabei gleich setzen
          beq       init06         * Wenn's 0 war, freie Kennung gefunden
          addq.w    #1,d4          * nchstes Bit prfen
          dbf       d5,init05

          moveq     #err_drv,d0    * Fehler! Kein Bit mehr frei fr uns!
          bra       abort          * Fehlerbehandlung und Ausgang

init06    move.w    d4,drive_id    * Laufwerkskennung speichern
          move.l    d0,drvbits     * Bitvektor fr's TOS abspeichern

          add.b     #'A',d4        * Laufwerkskennung zum Buchstaben machen
          move.b    d4,drive       * und in Signup-Meldung einsetzen

* Einbindung in die Vektoren BPB, MEDIACH und RWABS

          move.l    hdv_bpb,old_bpb * In BPB-Vektor einhngen
          move.l    #my_bpb,hdv_bpb

          move.l    hdv_rw,old_rw   * In RW-Vektor einhngen
          move.l    #my_rw,hdv_rw

          move.l    hdv_med,old_med * In Mediachange einhngen
          move.l    #my_med,hdv_med

* Signup ausgeben und eine Sekunde warten, damit der Benutzer die
* gewhlte Laufwerkskennung lesen kann.

          move.w    #signup,d0     * Signup Message ID
          bsr       message
          bsr       delay          * 1 Sekunde warten

* Fertig! Jetzt noch den Stack aufrumen und Supervisor abschalten.


          move.l    ssave,-(sp)    * Benutzer-Stack restaurieren
          move.w    #32,-(sp)      * SUPER
          trap      #1
          addq.l    #6,sp

* Damit das Ramdisk-Programm nicht berschrieben wird, mu es mit
* PTERMRES resident gemacht werden.

          move.w    #0,-(sp)       * Fehlercode 0 (OKAY) zurckliefern
          move.l    mysize,-(sp)   * Gre des reservierten Speichers          
          move.w    #49,-(sp)      * ptermres
          trap      #1             * Programm wird beendet.

* --------------------------------------------------------------------
*                             HILFSROUTINEN
* --------------------------------------------------------------------

* MESSAGE: Gibt die dem Result Code in D0 entsprechende Fehlermeldung
* aus.
*
* Wird MESSAGE mit einer positiven Fehlernummer in D0 aufgerufen, wird
* eine benutzerdefinierte Fehlermeldung aus der Tabelle USERERR entnommen.
* Bei negativen Fehlernummern wird die entsprechende Toolbox-Fehlermeldung
* aus der Tabelle TOOLERR angezeigt.
*

message   move.w    d0,-(sp)       * Fehlernummer sichern
          pea       errhead        * 'ERROR: ' Vorspann ausgeben
          move.w    #9,-(sp)       * print string
          trap      #1
          addq.l    #6,sp

          lea       usererr,a0     * User-Fehlermeldungen  fr D0>0
          move.w    (sp)+,d0       * Fehlerkennung zurckholen vom Stack
          bpl       msg01          * Falls D0>=0, weiter bei MSG01

          lea       toolerr,a0     * Toolbox-Meldungen fr D0<0
          neg.w     d0             * Nummer positiv machen

msg01     subq.w    #1,d0          * Fehlernummer dekrementieren
          beq       msg03          * bei 0 ist die richtige Meldung gefunden

msg02     tst.b     (a0)+          * suche nchstes 0-Byte
          bne       msg02
          bra       msg01          * Weiter, bis die Meldung gefunden wurde

msg03     move.l    a0,-(sp)       * Adresse des Fehlertextes
          move.w    #9,-(sp)       * print line
          trap      #1
          addq.l    #6,sp
          rts


* ABORT wird aufgerufen, falls bei der Installation der Ramdisk ein
* Fehler auftritt. Die entsprechende Fehlermeldung wird ausgegeben,
* auf einen Tastendruck gewartet, der Stack restauriert und das Programm
* wird beendet. Die Ramdisk wird dabei nicht resident im Speicher
* installiert.


abort     bsr       message        * Return Code auswerten
          bsr       delay          * Mindestens 1 Sekunde stehen lassen
          bsr       getkey         * Auf Tastendruck warten

          move.l    ssave,-(sp)    * Benutzer-Stack restaurieren
          move.w    #32,-(sp)      * SUPER
          trap      #1
          addq.l    #6,sp
          
          clr.w     -(sp)          * terminate process
          trap      #1             * so long and thanks for all the fish

* GETKEY wartet auf ein Zeichen. Das Ergebnis wird mit AND #$005F,D0
* verknpft, damit Kleinschrift zu Groschrift wird.

getkey    move.w    #8,-(sp)       * bconin w/o echo
          trap      #1
          addq.l    #2,sp
          and.w     #$005F,d0      * convert lowercase to uppercase
          rts

* DELAY wartet eine Sekunde, indem es auf den Tick Counter _HZ200 zugreift.

delay     move.l    hz200,d0       * 200 Hz Zhler
          add.l     #400,d0        * Plus 2 Sekunden
delay01   cmp.l     hz200,d0
          bcc       delay01        * warten, bis die Zeit vergeht...
          rts

 data                              * Weiter im DATA SEGMENT

dev_id    dc.w      3              * Gerteadresse, Default 3
drive_id  dc.w      8              * Laufwerkskennung, Default I: (Nr. 8)

bpb       dc.w      $200           * Bytes per Sector
          dc.w      2              * Sectors per Cluster
          dc.w      1024           * Bytes per Cluster
          dc.w      7              * Root Dir Sectors
          dc.w      5              * FAT length
          dc.w      6              * 2nd FAT start
          dc.w      18             * 1st data sector
bpb_size  dc.w      0              * Freie DATEN-Cluster (ohne FAT, DIR)
          dc.w      0              * 12bit FAT
          dc.l      0              * reserved

v30_start incbin    ramdisk.bin    * V30 Programmcode
v30_size  equ       *-v30_start    * Lnge des V30 Programms
v30_addr  equ       $03000000      * Startadresse 0300:0000

errhead   dc.b      13,10,10,'SC-RAMDISK: ',0

toolerr   dc.b      7,'Residente Toolbox SCTB.PRG nicht geladen!',0
          dc.b      7,'(reserved)',0
          dc.b      7,'SuperCharger nicht angeschlossen!',0
          dc.b      7,'TIMEOUT! SuperCharger antwortet nicht',0
          dc.b      7,'Protokollfehler',0
          dc.b      7,'ABIO Hotkey erkannt (kann mit ABIO.TOS '
          dc.b      'aktiviert werden)',0
          dc.b      7,'(reserved)',0
          dc.b      7,'Falscher Befehlscode!',0

usererr   dc.b      'Es sind bereits 32 Laufwerke aktiv, mehr geht nicht!'
          dc.b      0                       * message 2 follows
          dc.b      'ABIO-Hotkey luft noch. [A] = Ramdisk Abbruch, '
          dc.b      '[R] = Hotkey RESET ?'
          dc.b      0                       * message 3 follows
          dc.b      'SELECT: R = RETRY, A = ABORT ---> '
          dc.b      0                       * message 4 follows
          dc.b      'SuperCharger Ramdisk als Laufwerk '
drive     dc.b      '?: installiert.',0
          dc.b      'Warmstart, Ramdisk ist bereits im SuperCharger '
          dc.b      'installiert.',0


 bss

startadr  ds.l      1              * Startadresse bei Read/Write
sektor    ds.w      1              * Sektornummer
Anzahl    ds.w      1              * Anzahl der Sektoren

basepage  ds.l      1              * Adresse der Basepage
ssave     ds.l      1              * Benutzer-Stackpointer (USP)
mysize    ds.l      1              * Gre des reservierten Atari-Speichers


